local _, addon = ...
local TCL = addon.TomCatsLibs
local RarePanelToggle

local supportedMaps = {
    [14] = { name = "TomCats-ArathiHighlandsRares", title = "TomCat's Tours: Rares of Arathi Highlands" }, -- Arathi
    [62] = { name = "TomCats-DarkshoreRares", title = "TomCat's Tours: Rares of Darkshore" }, -- Darkshore
    [1355] = { name = "TomCats-Nazjatar", title = "TomCat's Tours: Nazjatar" }, -- Nazjatar
    [1462] = { name = "TomCats-Mechagon", title = "TomCat's Tours: Mechagon" }, -- Mechagon
}
local DISPLAY_STATE_CLOSED = 1
local DISPLAY_STATE_OPEN_MINIMIZED_NO_LOG = 2
local DISPLAY_STATE_OPEN_MINIMIZED_WITH_LOG = 3
local DISPLAY_STATE_OPEN_MAXIMIZED = 4
local DISPLAY_STATE_OPEN_MINIMIZED_WITH_RARE_LOG = 5

local lastDisplayStates = { DISPLAY_STATE_CLOSED, DISPLAY_STATE_CLOSED }
local function setLastDisplayState(displayState)
    lastDisplayStates[1] = lastDisplayStates[2]
    lastDisplayStates[2] = displayState
end


local function SidePanelToggle_Refresh()
    if TomCatsRareMapFrame:IsShown() or not QuestMapFrame:IsShown() then
        WorldMapFrame.SidePanelToggle.OpenButton:Show()
        WorldMapFrame.SidePanelToggle.CloseButton:Hide()
    else
        WorldMapFrame.SidePanelToggle.OpenButton:Hide()
        WorldMapFrame.SidePanelToggle.CloseButton:Show()
    end
end

TomCatsRareLogMixin = {}

function TomCatsRareLogMixin:InitLayoutIndexManager()
    self.layoutIndexManager = CreateLayoutIndexManager()
    self.layoutIndexManager:AddManagedLayoutIndex("RaresLog", QUEST_LOG_WAR_CAMPAIGN_LAYOUT_INDEX + 1)
    local faction = UnitFactionGroup("player")
    self.RaresFrame.Contents.WarCampaignHeader.Background:SetAtlas("Campaign_" .. faction)
    --    self.layoutIndexManager:AddManagedLayoutIndex("Other", QUEST_LOG_STORY_LAYOUT_INDEX + 1)
end

function TomCatsRareLogMixin:Refresh()
    local name, realm = UnitFullName("player")
    if not name then
        name = "Denizen of Azeroth"
    end
    self.RaresFrame.Contents.WarCampaignNextObjective.Text:SetText(
            "|cffffffffSomething very cool is coming soon!|r\n\nDear " .. name .. ",\n\nYou are hereby invited to vote on the next features to be added to this panel.  Please come visit me and look for the 'Twitch Picks' section to vote (address below). It's fast, easy, and best of all it's free!\n\nI'm looking forward to meeting you. As always, thank you for using my addons and for your support!\n\nSincerely,\n\nTomCat\nof TomCat's Tours\n\n|cffffffffhttps://twitch.tv/TomCat|r\n\nNotes on how this panel works:\n\nThe question mark button is for quests and the star button is for rares. I will remember which panel you last chose to have open while in one of the rares zones (i.e. Nazjatar, Mechagon, Arathi, Darkshore). Anywhere else, it's as if the panel was never there. Give it a try.  If something doesn't feel right, let me know!"
    )
    self.RaresFrame.Contents.WarCampaignHeader.Text:SetText("Rares Log")
    self.RaresFrame.Contents.WarCampaignHeader:Show()
    self.layoutIndexManager:Reset()
    self.RaresFrame.Contents:Layout()
end

function TomCatsRareLogMixin:UpdateDisplayState()
    if not WorldMapFrame:IsShown() then
        setLastDisplayState(DISPLAY_STATE_CLOSED)
    elseif QuestMapFrame:IsShown() or QuestMapFrame.DetailsFrame:IsShown() then
        if QuestMapFrame.DetailsFrame:IsShown() then
            self:Hide()
            QuestScrollFrame:Hide()
        else
            if supportedMaps[WorldMapFrame:GetMapID()] and addon.savedVariables.character.preferQuestLog == nil then
                self:Show()
                QuestScrollFrame:Hide()
                self:Refresh()
                setLastDisplayState(DISPLAY_STATE_OPEN_MINIMIZED_WITH_RARE_LOG)
                return
            end
            QuestScrollFrame:Show()
        end
        setLastDisplayState(DISPLAY_STATE_OPEN_MINIMIZED_WITH_LOG)
    else
        if WorldMapFrame:IsMaximized() then
            setLastDisplayState(DISPLAY_STATE_OPEN_MAXIMIZED)
        else
            setLastDisplayState(DISPLAY_STATE_OPEN_MINIMIZED_NO_LOG)
        end
    end
    self:Hide()
end

function TomCatsRareMapFrame_OnLoad(self)
    self:InitLayoutIndexManager()
end

function TomCatsRareMapFrame_OnEvent() end

function TomCatsRareMapFrame_OnHide() end

local function UpdateAll()
    TomCatsRareMapFrame:UpdateDisplayState()
    RarePanelToggle:Refresh()
    SidePanelToggle_Refresh()
end

local function RevertToQuestFrameShown(save)
    WorldMapFrame:SetDisplayState(DISPLAY_STATE_OPEN_MINIMIZED_WITH_LOG)
    QuestScrollFrame:Show()
    TomCatsRareMapFrame:Hide()
    RarePanelToggle:Refresh()
    SidePanelToggle_Refresh()
    setLastDisplayState(DISPLAY_STATE_OPEN_MINIMIZED_WITH_LOG)
    if save then
        SetCVar("questLogOpen", "1")
    end
end

local function Hook_WorldMapFrame_SidePanelToggle()
    if supportedMaps[WorldMapFrame:GetMapID()] then
        addon.savedVariables.character.preferQuestLog = true
    end
    if (lastDisplayStates[1] == DISPLAY_STATE_OPEN_MINIMIZED_WITH_RARE_LOG and lastDisplayStates[2] == DISPLAY_STATE_OPEN_MINIMIZED_NO_LOG) or
            (lastDisplayStates[2] == DISPLAY_STATE_OPEN_MINIMIZED_WITH_RARE_LOG and lastDisplayStates[1] == DISPLAY_STATE_OPEN_MINIMIZED_NO_LOG) then
        RevertToQuestFrameShown(true)
    end
end

local function Hook_ToggleQuestLog()
    if lastDisplayStates[2] == DISPLAY_STATE_OPEN_MINIMIZED_WITH_RARE_LOG then
        RevertToQuestFrameShown()
    end
end



TomCatsWorldMapRareSidePanelToggleMixin = { }

function TomCatsWorldMapRareSidePanelToggleMixin:OnClick()
    addon.savedVariables.character.preferQuestLog = nil
    if (self.OpenButton:IsShown()) then
        WorldMapFrame:SetDisplayState(DISPLAY_STATE_OPEN_MINIMIZED_WITH_LOG)
        QuestScrollFrame:Hide()
        TomCatsRareMapFrame:Show()
        TomCatsRareMapFrame:Refresh()
        self.OpenButton:Hide()
        self.CloseButton:Show()
        SetCVar("questLogOpen", "1")
    else
        WorldMapFrame:SetDisplayState(DISPLAY_STATE_OPEN_MINIMIZED_NO_LOG)
        TomCatsRareMapFrame:Hide()
        self.OpenButton:Show()
        self.CloseButton:Hide()
        SetCVar("questLogOpen", "0")
    end
    WorldMapFrame.SidePanelToggle.OpenButton:Show()
    WorldMapFrame.SidePanelToggle.CloseButton:Hide()
end

function TomCatsWorldMapRareSidePanelToggleMixin:Refresh()
    if WorldMapFrame.SidePanelToggle:IsShown() and supportedMaps[WorldMapFrame:GetMapID()] then
        self:Show()
        if TomCatsRareMapFrame:IsShown() then
            self.OpenButton:Hide()
            self.CloseButton:Show()
        else
            self.OpenButton:Show()
            self.CloseButton:Hide()
        end
    else
        self:Hide()
    end
end

local function ADDON_LOADED(_, _, arg1)
    if (arg1 == addon.name) then
        TCL.Events.UnregisterEvent("ADDON_LOADED", ADDON_LOADED)
        RarePanelToggle = WorldMapFrame:AddOverlayFrame("TomCatsWorldMapRareSidePanelToggleTemplate", "BUTTON", "BOTTOMRIGHT", WorldMapFrame:GetCanvasContainer(), "BOTTOMRIGHT", -2, 36)
        TomCatsRareMapFrame:SetParent(WorldMapFrame)
        TomCatsRareMapFrame:SetFrameStrata("HIGH")
        TomCatsRareMapFrame:SetFrameLevel(10)
        TomCatsRareMapFrame:ClearAllPoints()
        TomCatsRareMapFrame:SetPoint("TOPRIGHT", -6, -20)
        hooksecurefunc(WorldMapFrame, "SetMapID", UpdateAll)
        hooksecurefunc(WorldMapFrame.SidePanelToggle, "OnClick", Hook_WorldMapFrame_SidePanelToggle)
        hooksecurefunc(WorldMapFrame, "SetDisplayState", UpdateAll)
        hooksecurefunc("QuestMapFrame_ShowQuestDetails", UpdateAll)
        hooksecurefunc("ToggleQuestLog", Hook_ToggleQuestLog)
    end
end

TCL.Events.RegisterEvent("ADDON_LOADED", ADDON_LOADED)